#
# gen_file_if_different.mk
#
# Makefile macro for conditionally generating a prerequisite.  Use this
# when a prerequisite is generated dynamically, but dependent targets
# should not be rebuilt unless the prerequisite would be generated
# with different contents than previously generated.  Used in a two-pass
# Makefile.
#
# Copyright (c) 2015, 2016 First Silicon Designs, LLC.
#
# The "Software" in the following 2 paragraphs refers to this file containing
# the code to "gen_file_if_different" Make macro definition.
#
# The "Software" does NOT refer to any other files which you
# may have received alongside this file (e.g. as part of a larger project that
# incorporates gen_file_if_different).
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software, to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to permit
# persons to whom the Software is furnished to do so, subject to the following
# conditions:
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
# OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
# SOFTWARE.
#
#

# parameter 1 is a namespace, to allow multiple instances to be created
# parameter 2 is a category name, to allow multiple targets per namespace
# parameter 3 is the target file name with path if appropriate (safer)
# parameter 4 is the variable *name* which contains the file generator command
define gen_file_if_different =
.SECONDEXPANSION:
.PHONY: $(1)_$(2)_FILEGEN
$(1)_$(2)_FILEGEN : $(3).fresh $(3)
	@{ diff -q $$^ >/dev/null || cp $$^ ; } && rm $(3).fresh

# Use .DELETE_ON_ERROR: if you want to remove the "test" file when
# the generator program fails
#.DELETE_ON_ERROR:
$(3).fresh :
	@mkdir -p $$(@D)
	@$$($(4)) >> $$@ || { \
	   echo "ERROR: $(1)_$(2)_FILEGEN.fresh FAILED!" ; exit 2 ; }

# If the file does not exist, we need a rule to satisfy the prerequisite
# for the _FILEGEN rule above.  The empty file will cause the compare
# in the first rule to fail.  Later, this file might exist, but will
# only be overwritten if the new ".fresh" version is different.
$(3) :
	@mkdir -p $$(@D)
	@touch $$@

endef # gen_file_if_different
