#
# test_dyn_prereq.mk
#
# Test usage of dynamic prerequisite generation function
#

THIS_MAKEFILE := $(lastword $(MAKEFILE_LIST))

include gen_file_if_different.mk

# GENPROGRAM does not need to be defined prior to call.
# It will be expanded when the rule is executed.  Each call
# to the macro can use a different variable that contains
# the generator program.
#
ifeq ($(NEWPREREQ),1)
  GENPROGRAM := date
else
  GENPROGRAM := echo "This is the life!"
endif

# Example call.  Use macro args 1 and 2 to differentiate multiple calls.
# Notice the recursive call to make and the explicit naming of the
# makefile, to handle the case when it is not called "Makefile".  Make
# does not pass down the -f command line option when doing recursion.
#
ifneq ($(INCLUDES_BUILT),1)
$(eval $(call gen_file_if_different,name1,name2,targetfile1,GENPROGRAM))
all : name1_name2_FILEGEN
	$(MAKE) --no-print-directory -f $(THIS_MAKEFILE) \
                INCLUDES_BUILT=1 $(MAKECMDGOALS)
else
all : targetfile1
	@echo "Must have generated the dependent file!"
	@echo "Executing recipe for $@"
	touch $@
endif

# This rule is very helpful when debugging new Make rules
#
printvar-%:
	@echo '$*=$($@)'
